# $Id: TestFunctions.py 1049 2009-01-15 15:09:21Z graham $
#
# Unit testing for WebBrick library functions (Functions.py)
# See http://pyunit.sourceforge.net/pyunit.html
#

import sys
import os
import unittest
from string import upper

sys.path.append("../..")
from MiscLib.Functions import *

LibName = "MiscLib"

class TestFunctions(unittest.TestCase):
    def setUp(self):
        return

    def tearDown(self):
        return

    def doAssert(self, cond, msg):
        assert cond , msg

    # Actual tests follow

    def testNull(self):
        assert True, 'Null test failed'

    def testSuff1(self):
        assert endsWith("xxx","xxx")

    def testSuff2(self):
        assert endsWith("yyyzzz","zzz")

    def testSuff3(self):
        assert not endsWith("xxxyyy","xxx")

    def testCwd(self):
        # Check test environment current directory)
        cwd = os.getcwd() 
        assert endsWith(cwd, LibName+os.sep+"tests"), "Cwd is " + cwd

    def testConcatMap1(self):
        assert concatMap(upper,["a","B","cDe","FgH"])=="ABCDEFGH"

    def testConcatMap2(self):
        assert concatMap(upper,["a"])=="A"

    def testConcatMap3(self):
        assert concatMap(upper,[])==""

    def testFstSnd1(self):
        pair=('a','b')
        assert (fst(pair)=='a') and (snd(pair)=='b')

    def testFstSnd2(self):
        pair=(1,None)
        assert (fst(pair)==1) and (snd(pair)==None)

    def testIterAll1(self):
        seq = ('a',2,'c')
        seqall = iterAll(seq)
        assert seqall.next()=='a'
        assert seqall.next()==2
        assert seqall.next()=='c'
        assert seqall.next()==None
        assert seqall.next()==None
        assert seqall.next()==None

    def testIterAll2(self):
        seq = ('a',2,'c')
        seqall = iterAll(seq,"foo")
        assert seqall.next()=='a'
        assert seqall.next()==2
        assert seqall.next()=='c'
        assert seqall.next()=="foo"
        assert seqall.next()=="foo"
        assert seqall.next()=="foo"

    def testZipAll1(self):
        # z = tuple( [x for x in zipAll((1,2,3), (4,5))] )
        z = tuple( zipAll((1,2,3), (4,5)) )
        e = ((1,4),(2,5),(3,None))
        assert z==e, z

    def testZipAll2(self):
        z = tuple( zipAll((1,2,3), (4,5), (6,7,8,9), ()) )
        e = ((1,4,6,None), (2,5,7,None), (3,None,8,None), (None,None,9,None))
        assert z==e, z

    def testAll1(self):
        assert(all(isEq(1),[1,1,1,1]))

    def testAll2(self):
        assert(not all(isEq(1),[1,1,0,1]))

    def testAny1(self):
        assert(any(isEq(1),[1,2,3,4]))

    def testAny2(self):
        assert(not any(isEq(0),[1,2,3,4]))

    def testAllEq1(self):
        assert allEq(2,[2,2,2,2])

    def testAllEq2(self):
        assert not allEq(2,[2,2,0,2])

    def testAllEq3(self):
        assert allEq(2,[])

    def testAllNe1(self):
        assert allNe(2,[0,1,3,4])

    def testAllNe2(self):
        assert not allNe(2,[0,1,2,3,4])

    def testAllNe3(self):
        assert allNe(2,[])

    def testFilterSplit1(self):
        def isOdd(v): return isEq(1)(v&1)
        vs = [1,2,3,4,5,6,7,5,3,2,1]
        (v1,v2) = filterSplit(isOdd,vs)
        assert v1 == [1,3,5,7,5,3,1], str(v1)
        assert v2 == [2,4,6,2]

    def testFilterSplit2(self):
        vs = [1,2,3,4,5,6,7,5,3,2,1]
        (v1,v2) = filterSplit(isEq(9),vs)
        assert v1 == []
        assert v2 == [1,2,3,4,5,6,7,5,3,2,1]

    def testFilterSplit3(self):
        vs = [1,2,3,4,5,6,7,5,3,2,1]
        (v1,v2) = filterSplit(isNe(9),vs)
        assert v1 == [1,2,3,4,5,6,7,5,3,2,1]
        assert v2 == []

    def testFormatIntList1(self):
        return self.assertEqual(
            "1,2,3,4,10,15,255",
            formatIntList([1,2,3,4,10,15,255]))

    def testFormatIntList2(self):
        return self.assertEqual(
            "01:02:03:04:0A:0F:FF",
            formatIntList([1,2,3,4,10,15,255],":",formatInt("%02X")))

    def testFormatList1(self):
        txt = formatList(('a','b','c'))
        assert txt=="( 'a'\n, 'b'\n, 'c'\n)", repr(txt)

    def testFormatList2(self):
        txt = formatList(('a','b','c'),left=2)
        assert txt=="( 'a'\n  , 'b'\n  , 'c'\n  )", repr(txt)

    def testFormatList3(self):
        txt = formatList(('a','b','c'),left=2,right=40)
        assert txt=="('a', 'b', 'c')", repr(txt)

    def testFormatList4(self):
        txt = formatList(('a','b',('c','d','e')),left=2,right=20)
        assert txt=="( 'a'\n  , 'b'\n  , ('c', 'd', 'e')\n  )", repr(txt)

    def testFormatDict1(self):
        txt = formatDict({'a':'foo','b':'bar'})
        assert txt=="{ 'a': 'foo'\n, 'b': 'bar'\n}", repr(txt)

    def testFormatDict9(self):
        dic = \
            { 'style': None
            , 'elems': 
              ( { 'Control': 
                  {'Selection': {'Values': {'Wb:Off': '0', 'Wb:On': '1'}, 'ctype': 'Wb:SwitchToggle'}}
                , 'source': {'endpoint': 'WebBrick2', 'channel': 'DO2'}
                , 'style': None
                , 'target': {'endpoint': 'WebBrick2', 'channel': 'DI1'}
                , 'label': None
                }
              ,
              )
            , 'label': 'loose'
            }
        exp = (
            "{ 'style': None\n"+
            ", 'elems': \n"+
            "  ( { 'Control': \n"+
            "      { 'Selection': \n"+
            "        { 'Values': {'Wb:Off': '0', 'Wb:On': '1'}\n"+
            "        , 'ctype': 'Wb:SwitchToggle'\n"+
            "        }\n"+
            "      }\n"+
            "    , 'source': \n"+
            "      {'endpoint': 'WebBrick2', 'channel': 'DO2'}\n"+
            "    , 'style': None\n"+
            "    , 'target': \n"+
            "      {'endpoint': 'WebBrick2', 'channel': 'DI1'}\n"+
            "    , 'label': None\n"+
            "    }\n"+
            "  )\n"+
            ", 'label': 'loose'\n"+
            "}" )
        txt = formatDict(dic,left=0,right=60,pos=0)
        assert txt==exp, "\n"+txt # "\n"+txt  # repr(txt)

    def testCompareLists1(self):
        assert compareLists([],[])==None

    def testCompareLists2(self):
        assert compareLists(['a'],['a'])==None

    def testCompareLists3(self):
        assert compareLists(None,[])==None

    def testCompareLists4(self):
        assert compareLists(['a','b','c'],['d','e','f'])==(['a','b','c'],['d','e','f'])

    def testCompareLists5(self):
        assert compareLists(['a','b','c'],['b','d','c','e'])==(['a'],['d','e'])

    def testCompareLists6(self):
        assert compareLists(['a','b','a'],['b','d','b','d'])==(['a','a'],['d','d'])

    def testCompareLists7(self):
        assert compareLists(['a','b','c'],None)==(['a','b','c'],[])

    def testCompareDicts1(self):
        assert compareDicts({},{})==None

    def testCompareDicts2(self):
        assert compareDicts({'foo':'bar'},{'foo':'bar'})==None

    def testCompareDicts3(self):
        difs = compareDicts({'a':'foo','b':'bar'},{'a':'foo'})
        assert difs==({'b':'bar'},{}), repr(difs)

    def testCompareDicts4(self):
        assert compareDicts({'a':'foo','b':'bar'},{'a':'foo','b':'baz'})==({'b':'bar'},{'b':'baz'})

    def testCompareDicts5(self):
        assert compareDicts({'a':'foo','b':{'c':'bar'}},{'a':'foo','b':{'c':'bar'}})==None

    def testCompareDicts6(self):
        d1 = \
            { 'elems': 
              ( { 'Control': 
                  { 'source': {'endpoint': 'WebBrick2', 'channel': 'DO2'}
                  , 'Selection': {'Values': {'Wb:Off': '0', 'Wb:On': '1'}, 'ctype': 'Wb:SwitchToggle'}
                  , 'target': {'endpoint': 'WebBrick2', 'channel': 'DI1'}
                  }
                }
              ,
              )
            , 'label': 'Shower light'
            }
        d2 = \
            { 'style': None
            , 'elems': 
              ( { 'Control': 
                  {'Selection': {'Values': {'Wb:Off': '0', 'Wb:On': '1'}, 'ctype': 'Wb:SwitchToggle'}}
                , 'source': {'endpoint': 'WebBrick2', 'channel': 'DO2'}
                , 'style': None
                , 'target': {'endpoint': 'WebBrick2', 'channel': 'DI1'}
                , 'label': None
                }
              ,
              )
            , 'label': 'loose'
            }
        dif1 = \
            { 'elems': 
                ( { 'Control': 
                    { 'source': {'endpoint': 'WebBrick2', 'channel': 'DO2'}, 
                      'target': {'endpoint': 'WebBrick2', 'channel': 'DI1'}
                    }
                  }
                ,
                )
            , 'label': 'Shower light'
            }
        dif2 = \
            { 'elems': 
              ( { 'source': {'endpoint': 'WebBrick2', 'channel': 'DO2'}
                , 'target': {'endpoint': 'WebBrick2', 'channel': 'DI1'}
                }
              ,
              )
            , 'label': 'loose'
            }
        (dd1,dd2) = compareDicts(d1,d2)
        assert dd1==dif1, "dif1:\n"+formatDict(dd1,0,60,0)
        assert dd2==dif2, "dif2:\n"+formatDict(dd2,0,60,0)
        #assert (dd1,dd2)==(dif1,dif2),"dif1:\n"+formatDict(dd1,0,60,0)+"\ndif2:\n"+formatDict(dd2,0,60,0)

    def testCompareDicts7(self):
        d1 = \
            { 'group': 'loose'
            , 'elems':
              ( { 'Control':
                  { 'Selection':
                    { 'Values': {'Wb:Off': '0', 'Wb:On': '1'}
                    , 'ctype': 'Wb:SwitchToggle'
                    }
                  }
                , 'source':
                  {'endpoint': 'WebBrick2', 'channel': 'DO2'}
                , 'target':
                  {'endpoint': 'WebBrick2', 'channel': 'DI1'}
                }
              ,
              )
            , 'label': 'Shower light'
            }
        d2 = \
            { 'style': None
            , 'group': 'loose'
            , 'elems':
              ( { 'Control':
                  { 'Selection':
                    { 'Values': {'Wb:Off': '0', 'Wb:On': '1'}
                    , 'ctype': 'Wb:SwitchToggle'
                    }
                  }
                , 'source':
                  {'endpoint': 'WebBrick2', 'channel': 'DO2'}
                , 'style': None
                , 'target':
                  {'endpoint': 'WebBrick2', 'channel': 'DI1'}
                , 'label': None
                }
              ,
              )
            , 'label': 'Shower light'
            }
        dd = compareDicts(d1,d2)
        if dd != None:
            (dd1,dd2) = dd
            assert dd1==None, "dd1:\n"+formatDict(dd1,0,60,0)
            assert dd2==None, "dd2:\n"+formatDict(dd2,0,60,0)

    def testCompareDicts8(self):
        d1 = \
            { 'group': 'loose'
            , 'elems':
              [ { 'Control':
                  { 'Selection':
                    { 'Values': {'Wb:Off': '0', 'Wb:On': '1'}
                    , 'ctype': 'Wb:SwitchToggle'
                    }
                  }
                , 'source':
                  {'endpoint': 'WebBrick2', 'channel': 'DO2'}
                , 'target':
                  {'endpoint': 'WebBrick2', 'channel': 'DI1'}
                }
              ]
            , 'label': 'Shower light'
            }
        d2 = \
            { 'style': None
            , 'group': 'loose'
            , 'elems':
              [ { 'Control':
                  { 'Selection':
                    { 'Values': {'Wb:Off': '0', 'Wb:On': '1'}
                    , 'ctype': 'Wb:SwitchToggle'
                    }
                  }
                , 'source':
                  {'endpoint': 'WebBrick2', 'channel': 'DO2'}
                , 'style': None
                , 'target':
                  {'endpoint': 'WebBrick2', 'channel': 'DI1'}
                , 'label': None
                }
              ]
            , 'label': 'Shower light'
            }
        dd = compareDicts(d1,d2)
        if dd != None:
            (dd1,dd2) = dd
            assert dd1==None, "dd1:\n"+formatDict(dd1,0,60,0)
            assert dd2==None, "dd2:\n"+formatDict(dd2,0,60,0)

# Code to run unit tests directly from command line.
# Constructing the suite manually allows control over the order of tests.
def getTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(TestFunctions("testNull"))
    suite.addTest(TestFunctions("testSuff1"))
    suite.addTest(TestFunctions("testSuff2"))
    suite.addTest(TestFunctions("testSuff3"))
    suite.addTest(TestFunctions("testCwd"))
    suite.addTest(TestFunctions("testConcatMap1"))
    suite.addTest(TestFunctions("testConcatMap2"))
    suite.addTest(TestFunctions("testConcatMap3"))
    suite.addTest(TestFunctions("testFstSnd1"))
    suite.addTest(TestFunctions("testFstSnd2"))
    suite.addTest(TestFunctions("testIterAll1"))
    suite.addTest(TestFunctions("testIterAll2"))
    suite.addTest(TestFunctions("testZipAll1"))
    suite.addTest(TestFunctions("testZipAll2"))
    suite.addTest(TestFunctions("testAll1"))
    suite.addTest(TestFunctions("testAll2"))
    suite.addTest(TestFunctions("testAny1"))
    suite.addTest(TestFunctions("testAny2"))
    suite.addTest(TestFunctions("testAllEq1"))
    suite.addTest(TestFunctions("testAllEq2"))
    suite.addTest(TestFunctions("testAllEq3"))
    suite.addTest(TestFunctions("testAllNe1"))
    suite.addTest(TestFunctions("testAllNe2"))
    suite.addTest(TestFunctions("testAllNe3"))
    suite.addTest(TestFunctions("testFilterSplit1"))
    suite.addTest(TestFunctions("testFilterSplit2"))
    suite.addTest(TestFunctions("testFilterSplit3"))
    suite.addTest(TestFunctions("testFormatIntList1"))
    suite.addTest(TestFunctions("testFormatIntList2"))
    suite.addTest(TestFunctions("testFormatList1"))
    suite.addTest(TestFunctions("testFormatList2"))
    suite.addTest(TestFunctions("testFormatList3"))
    suite.addTest(TestFunctions("testFormatList4"))
    suite.addTest(TestFunctions("testFormatDict1"))
    suite.addTest(TestFunctions("testFormatDict9"))
    suite.addTest(TestFunctions("testCompareLists1"))
    suite.addTest(TestFunctions("testCompareLists2"))
    suite.addTest(TestFunctions("testCompareLists3"))
    suite.addTest(TestFunctions("testCompareLists4"))
    suite.addTest(TestFunctions("testCompareLists5"))
    suite.addTest(TestFunctions("testCompareLists6"))
    suite.addTest(TestFunctions("testCompareLists7"))
    suite.addTest(TestFunctions("testCompareDicts1"))
    suite.addTest(TestFunctions("testCompareDicts2"))
    suite.addTest(TestFunctions("testCompareDicts3"))
    suite.addTest(TestFunctions("testCompareDicts4"))
    suite.addTest(TestFunctions("testCompareDicts5"))
    suite.addTest(TestFunctions("testCompareDicts6"))
    suite.addTest(TestFunctions("testCompareDicts7"))
    suite.addTest(TestFunctions("testCompareDicts8"))
    return suite
    # suite = unittest.makeSuite(WidgetTestCase,'test')

if __name__ == "__main__":
    # unittest.main()
    runner = unittest.TextTestRunner()
    runner.run(getTestSuite())

# End.



"""
Simple SPARQL HTTP protocol client
"""

import os, os.path
import sys

import re
import logging
import httplib
import urllib
import urlparse
try:
    import json
except ImportError:
    # Running Python 2.5 with simplejson?
    import simplejson as json

logger = logging.getLogger(__name__)

class SparqlHttpClient(object):
    """
    Class implements simple SPARQL HTTP protocol client
    """
    def __init__(self, endpointhost="localhost:3030", endpointpath="/ds", endpointuri=None):
        # Default SPARQL endpoint details based on Fuseki defaults
        self._endpointhost = None
        self._endpointpath = None
        self._endpointuri  = None
        self.setQueryEndpoint(endpointhost, endpointpath, endpointuri)
        return

    def setQueryEndpoint(self, endpointhost=None, endpointpath=None, endpointuri=None):
        if endpointuri:
            # assume scheme is http, no query, no fragment
            up = urlparse.urlsplit(endpointuri)
            endpointhost = up.netloc
            endpointpath = up.path
        if endpointhost: self._endpointhost = endpointhost
        if endpointpath: self._endpointpath = endpointpath
        logger.debug("setQueryEndPoint: endpointhost %s: " % self._endpointhost)
        logger.debug("setQueryEndPoint: endpointpath %s: " % self._endpointpath)

    def doQueryGET(self, query, accept="application/JSON", JSON=True):
        """
        Issue SPARQL query as HTTP GET request.
        """
        ###print "---- query "+query
        reqheaders   = {
            "Accept":       accept
            }
        hc = httplib.HTTPConnection(self._endpointhost)
        encodequery  = urllib.urlencode({"query": query})
        hc.request("GET", self._endpointpath+"?"+encodequery, None, reqheaders)
        response = hc.getresponse()
        status = response.status
        reason = response.reason
        responsedata = response.read()
        hc.close()
        ###print "---- responsedata "+responsedata
        if JSON:
            responsedata = json.loads(responsedata)
        return ((status, reason), responsedata)

    def doQueryPOST(self, query, accept="application/JSON", JSON=True):
        """
        Issue SPARQL query as HTTP POST request.
        """
        ###print "---- query "+query
        reqheaders   = {
            "Content-type": "application/x-www-form-urlencoded",
            "Accept":       accept
            }
        hc = httplib.HTTPConnection(self._endpointhost)
        encodequery  = urllib.urlencode({"query": query})
        hc.request("POST", self._endpointpath, encodequery, reqheaders)
        response = hc.getresponse()
        status = response.status
        reason = response.reason
        responsedata = response.read()
        hc.close()
        ###print "---- responsedata "+responsedata
        if JSON:
            responsedata = json.loads(responsedata)
        return ((status, reason), responsedata)

# End.



# $Id: TestScanFiles.py 1058 2009-01-26 10:39:19Z graham $
#
# Unit testing for WebBrick library functions (Functions.py)
# See http://pyunit.sourceforge.net/pyunit.html
#

import sys
import unittest
import re
from os.path import normpath

sys.path.append("../..")
from MiscLib.ScanFiles import *
from MiscLib.Functions import compareLists

class TestScanFiles(unittest.TestCase):
    def setUp(self):
        self.testpath = "resources/"
        self.testpatt = re.compile( r'^TestScanFiles.*\.txt$' )
        return

    def tearDown(self):
        return

    # Actual tests follow

    def testCollectShallow(self):
        files    = CollectFiles(self.testpath,self.testpatt,recursive=False)
        expected = [ (self.testpath,"TestScanFiles1.txt")
                   , (self.testpath,"TestScanFiles2.txt")
                   ]
        c = compareLists(files, expected)
        assert c == None, "Wrong file list: "+repr(c)

    def testCollectRecursive(self):
        files    = CollectFiles(self.testpath,self.testpatt)
        expected = [ (self.testpath,"TestScanFiles1.txt")
                   , (self.testpath,"TestScanFiles2.txt")
                   , (self.testpath+"TestScanFilesSubDir","TestScanFiles31.txt")
                   , (self.testpath+"TestScanFilesSubDir","TestScanFiles32.txt")
                   ]
        c = compareLists(files, expected)
        assert c == None, "Wrong file list: "+repr(c)

    def testJoinDirName(self):
        # normpath used here to take care of dir separator issues.
        n = joinDirName("/root/sub","name")
        assert n==normpath("/root/sub/name"), "JoinDirName failed: "+n
        n = joinDirName("/root/sub/","name")
        assert n==normpath("/root/sub/name"), "JoinDirName failed: "+n
        n = joinDirName("/root/sub/","/name")
        assert n==normpath("/name"), "JoinDirName failed: "+n

    def testReadDirNameFile(self):
        assert readDirNameFile(self.testpath,"TestScanFiles1.txt"), "Read dir,file 'TestScanFiles1.txt' failed"

    def testReadFile(self):
        assert readFile(self.testpath+"TestScanFiles1.txt"), "Read file 'TestScanFiles1.txt' failed"


# Code to run unit tests directly from command line.
# Constructing the suite manually allows control over the order of tests.
def getTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(TestScanFiles("testCollectShallow"))
    suite.addTest(TestScanFiles("testCollectRecursive"))
    suite.addTest(TestScanFiles("testJoinDirName"))
    suite.addTest(TestScanFiles("testReadDirNameFile"))
    suite.addTest(TestScanFiles("testReadFile"))
    return suite

if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(getTestSuite())
    


#!/usr/bin/env python

"""
Context manager for redirecting stdin to a supplied file or stream
"""

import sys

class SwitchStdin:
    """
    Context handler class that swiches standard input to a named file or supplied stream.
    """
    
    def __init__(self, fileorstr):
        self.fileorstr = fileorstr
        self.opened = False
        return
    
    def __enter__(self):
        if isinstance(self.fileorstr,basestring):
            self.inpstr = open(self.fileorstr, "r")
            self.opened = True
        else:
            self.inpstr = self.fileorstr
        self.savestdin = sys.stdin
        sys.stdin = self.inpstr
        return self.inpstr

    def __exit__(self, exctype, excval, exctraceback):
        if self.opened: self.inpstr.close()
        sys.stdin = self.savestdin
        return False

if __name__ == "__main__":
    import StringIO
    inpstr = StringIO.StringIO("Hello world")
    with SwitchStdin(inpstr) as mystdin:
        inptxt = mystdin.read()
    print repr(inptxt)
    assert inptxt == "Hello world"
    inpstr.close()

# End.



# $Id: DomHelpers.py 1058 2009-01-26 10:39:19Z graham $
#
# Copyright (c) 2008 WebBrick Systems Limited
# Released under the MIT licence
# See LICENCE.TXT included with these files,
# or http://www.opensource.org/licenses/mit-license.php
#

import types
from os.path import exists
from os import rename, remove, linesep

###OLD CODE:
###from xml.dom.ext.reader import Sax2
###from xml.dom.ext import Print, PrettyPrint
###END

from xml.dom import minidom

from StringIO  import StringIO
from string import strip

from Functions import concatMap

"""
Miscellaneous DOM helper functions.
"""

def parseXmlString(config):
    """
    Parse XML from a supplied string into a DOM structure
    """
    if type(config) == types.UnicodeType:
        s = StringIO(config.encode("utf-8") )
    else:
        s = StringIO(config)
    return parseXmlStream(s)

def parseXmlFile(name):
    """
    Read and parse XML from a file into a DOM structure
    """
    f = open(name,"r")
    dom = parseXmlStream(f)
    f.close()
    return dom

def saveXmlToFile(name, xmlDom, doBackup=True):
    """
    write the Xml DOM to a disk file.
    """
    if doBackup:
        if exists(name):
            if exists(name+'.bak'):
                remove(name+'.bak')
            rename( name, name+'.bak')
    f = open(name,"w")
    xmlDom.writexml(f)
    ###OLD CODE:
    ###Print(xmlDom, f)
    ###END
    f.close()

def saveXmlToFilePretty(name, xmlDom, doBackup=True):
    """
    write bthe Xml DOM to a disk file.
    """
    if doBackup:
        if exists(name):
            if exists(name+'.bak'):
                remove(name+'.bak')
            rename( name, name+'.bak')
    f = open(name,"w")
    xmlDom.writexml(f, addindent="  ", newl="\n" )
#    xmlDom.writexml(f, addindent="  ", newl=linesep )
    ###OLD CODE:
    ###Print(xmlDom, f)
    ###END
    f.close()

def parseXmlStream(inpstr):
    """
    Parse XML from a supplied stream into a DOM structure.
    Closes the stream when done.
    """
    dom = minidom.parse(inpstr)
    inpstr.close()
    ###OLD CODE:
    #### See: http://pyxml.sourceforge.net/topics/howto/node18.html
    ###reader = Sax2.Reader()
    ###dom    = reader.fromStream(configstr)
    ###configstr.close()
    ###END
    return dom

def getElemXml(node):
    """
    Return text value of xml element (include all its children in the Xml string)
    """
    return node.toxml(encoding="utf-8")
    ###OLD CODE:
    ###s = StringIO()
    ###Print(node, s)
    ###result = s.getvalue()
    ###s.close()
    ###return result
    ###END

def getElemPrettyXml(node):
    """
    Return prettyfied text value of xml element 
    (include all its children in the Xml string)
    """
    return node.toprettyxml(indent="  ", encoding="utf-8")
    ###OLD CODE:
    #s = StringIO()
    ## See: http://infohost.nmt.edu/tcc/help/pubs/pyxml/printing.html
    #PrettyPrint( node, stream=s, encoding='UTF-8', indent='  ', preserveElements=None )
    #result = s.getvalue()
    #s.close()
    #return result
    ###END

def getAttrText(elem,attr):
    """
    Return text value from a named attribute of the supplied element node.
    """
    return elem.getAttribute(attr)

def getElemText(elem):
    """
    Return text value from a single element node.
    This is a concatenation of text nodes contained within the element.
    """
    return getNodeListText(elem.childNodes)

def getNodeListText(nodelist):
    """
    Return concatenation of text values from a supplied list of nodes
    """
    rc = u""
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return rc

def getFirstTextNodeValue(elem):
    """
    Return text value from the first named subnode of the supplied node.
    This is only the text at that node and 
    """
    return getElemText(elem)

#    for node in elem.childNodes:
#        if ( node.nodeType == node.TEXT_NODE ):
#            return node.nodeValue
#    return ""

def getNamedElem(parent, nodename):
    """
    Return first subnode of given parent with given tag name, or None
    if there is no such subnode.
    """
    nodes = parent.getElementsByTagName(nodename)
    if nodes:
        return nodes[0]
    return None

#TODO: deprecate me
def getNamedNode(node, nodename):
    """
    Return first named subnode of the supplied node
    """
    subnodes = node.getElementsByTagName(nodename)
    if len(subnodes) > 0:
        return subnodes[0]
    return None

def getNamedNodeXml(parent, nodename):
    """
    Return Xml string of the first named subnode 
    (include all its children in the Xml string),
    or None.
    """
    subnode = getNamedElem(parent, nodename)
    if subnode:
        return getElemXml(subnode)
    return None
   
def getNamedNodeText(parent, nodename):
    """
    Return text value from the first named subnode of the supplied node
    This is a concatenation of text nodes contained within the first named element.
    """
    subnode = getNamedElem(parent, nodename)
    return subnode and getElemText(subnode)

def getNamedNodeAttrText(node, nodename, attr):
    """
    Return text value from a named attribute of the first named subnode of the supplied node
    """
    elem = getNamedElem(node, nodename)
    return elem and elem.getAttribute(attr)

# Node content replacement

def removeChildren(elm):
    while elm.hasChildNodes():
        elm.removeChild(elm.firstChild)
    return elm

def replaceChildren(elm,newchildren):
    removeChildren(elm)
    for n in newchildren:
        elm.appendChild(n)
    return elm

def replaceChildrenText(elm,newtext):
    replaceChildren(elm, [elm.ownerDocument.createTextNode(newtext)])
    return elm

# Node test functions

def isElement(node):
    """
    Returns True if the supplied node is an element
    """
    return node.nodeType == node.ELEMENT_NODE

def isAttribute(node):
    """
    Returns True if the supplied node is an attribute
    """
    return node.nodeType == node.ATTRIBUTE_NODE

def isText(node):
    """
    Returns True if the supplied node is free text.
    """
    return node.nodeType == node.TEXT_NODE

# Content manipulation helpers
def escapeChar(c,d={}):
    if c == '<': return "&lt;"
    if c == '>': return "&gt;"
    if c == '&': return "&amp;"
    return d.get(c,c)

def escapeCharForHtml(c):
    return escapeChar(c,d={'\n': "<br/>"})

def escapeText(s):
    return concatMap(escapeChar, s)

def escapeTextForHtml(s):
    return concatMap(escapeCharForHtml, s)

def getDictFromXml(elm):
    """
    Turn an Xml DOM into a python dictionary, this only allows for a simple XML structure that consists only of 
    attributes, the element text and nested elements.  The attributes are stored in the dictionary by name.

    Nested elements are stored as dictionaries keyed by element tag.  The element text body is stored in the dictionary with a
    blank name, i.e. an empty string. Only text nodes, element nodes and element attributes are understood.

    Also this code will create a list of dictionaries where an element contains elements whose name is the singular
    form of its own name, i.e. the parent element is the child element name plus an 's'. Note in this case all
    other child elements are ignored.

    This code is an interim measure as we shift to using python data structures for configuration, instead of XML.

    TODO should this code use from xml.sax.saxutils import escape, unescape
    """
    nodes = None
    if elm.nodeName.endswith( 's' ):
        nodes = elm.getElementsByTagName(elm.nodeName[0:-1])

    if nodes:
        result = list()
        for node in nodes:
            result.append( getDictFromXml(node) )
    else:
        result = dict()
        # for text body of elm. Only if contains some non white space
        txt = strip(getElemText(elm))
        if txt:
            # attempt convert to numeric
#            try:
#                result[""] = int(txt)
#            except ValueError:
            result[""] = txt

        # for each attribute in elm
        if elm.attributes:
            for idx in range(elm.attributes.length):
                attr = elm.attributes.item(idx)
                # attempt convert to numeric
#                try:
#                    result[attr.name] = int(attr.value)
#                except ValueError:
                result[attr.name] = attr.value

        # for each child element in elm
        for node in elm.childNodes:
            if isElement(node):
                newVal = getDictFromXml(node)
                if result.has_key(node.nodeName):
                    # turn into list, for now skip.
                    if not isinstance( result[node.nodeName], list ):
                        ntry = result[node.nodeName]
                        result[node.nodeName] = list()
                        result[node.nodeName].append( ntry )
                    result[node.nodeName].append( newVal )
                else:
                    result[node.nodeName] = newVal
        # LPK CHANGE. An element always creates a dictionary. If there is
        # only a text entry then the dictionary has a single key of '' (the empty string)
        # flatten elements that only have content
#        if len(result) == 1 and result.has_key(''):
#            result = result['']

    return result

def getDictFromXmlString( str ):
    return getDictFromXml( parseXmlString( str ) )

def getDictFromXmlFile( fileName ):
    try :
        return getDictFromXml( parseXmlFile( fileName ) )
    except IOError, ex:  # file does not exist
        return None

def getXmlElemFromObject( doc, addTo, obj ):
    """
    Create correct element type for obj
    """
    if isinstance( obj, dict ):
        # Add multiple elements.
        for k in obj:
            # create element and recurse
            if k == '': 
                # empty string key, TEXT node.
                elm = doc.createTextNode( unicode(obj[k]) )
                addTo.appendChild( elm )

            elif isinstance( obj[k], list ):
                # Add multiple elements. Each value should be a dictionary.
                for val in obj[k]:
                    elm = doc.createElement( k )
                    addTo.appendChild( elm )
                    getXmlElemFromObject( doc, elm, val )

            elif isinstance( obj[k], dict ):
                # element
                elm = doc.createElement( k )
                addTo.appendChild( elm )
                getXmlElemFromObject( doc, elm, obj[k] )

            elif isinstance( obj[k], basestring ):
                # attribute
                atr = doc.createAttribute( k )
                addTo.setAttribute( k, unicode(obj[k]) )

            else:
                # should not get here as Dictionary form has strings, lists and dictionaries in it.
                elm = doc.createElement( k )
                addTo.appendChild( elm )
                getXmlElemFromObject( doc, elm, obj[k] )

    else:
        # add single text element.
        # Should not get here as obj should be dictionary
        # adding attributes and elements.
        elm = doc.createTextNode( unicode(obj) )
        addTo.appendChild( elm )

def getXmlDomFromDict( dct, rootElem = None ):
    """
    rootElem is the name for the root element, if None then the dct must have a 
    single root level element.
    """
    from xml.dom.minidom import getDOMImplementation
    impl = getDOMImplementation()

    if rootElem:
        newdoc = impl.createDocument(None, rootElem, None)
        getXmlElemFromObject( newdoc, newdoc.documentElement, dct )
    else:
        k1 = dct.keys()[0]
        newdoc = impl.createDocument(None, k1, None)
        getXmlElemFromObject( newdoc, newdoc.documentElement, dct[k1] )

    return newdoc

# End.



# $Id: TestCombinators.py 1047 2009-01-15 14:48:58Z graham $
#
# Unit testing for WebBrick library combinators
# See http://pyunit.sourceforge.net/pyunit.html
#

import sys
import unittest

sys.path.append("../..")
from MiscLib.Combinators import *

class TestCombinators(unittest.TestCase):

    def setUp(self):
        return

    def tearDown(self):
        return

    # Test cases

    def testApply(self):
        # Is function application like BCPL?  (fn can be a variable)
        def ap(f,v): return f(v)
        def inc(n): return n+1
        assert ap(inc,2)==3

    def testCurry(self):
        def f(a,b,c): return a+b+c
        g = curry(f,1,2)
        assert g(3) == 6

    def testCompose(self):
        def f(a,b,c): return a+b+c
        def g(a,b):   return a*b
        h = compose(f,g,1000,200)
        assert h(3,4) == 1212, "h(3,4) is "+str(h(3,4))

# Code to run unit tests directly from command line.
def getTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(TestCombinators("testApply"))
    suite.addTest(TestCombinators("testCurry"))
    suite.addTest(TestCombinators("testCompose"))
    return suite

if __name__ == "__main__":
    # unittest.main()
    runner = unittest.TextTestRunner()
    runner.run(getTestSuite())



# $Id: Functions.py 1047 2009-01-15 14:48:58Z graham $
#
"""
Miscellaneous functions for use with Python code, 
mostly based on Haskell library elements.
"""

from operator import concat, and_, or_

def concatMap(f,vs):
    """
    Map function over list and concatenate results.
    """
    return reduce( concat, map (f, vs), "")

def fst((a,b)):
    """
    First element of a pair.
    """
    return a

def snd((a,b)):
    """
    Second element of a pair.
    """
    return b

def iterAll(c,sentinel=None):
    """
    Like the built-in 'iter' function, except that when the supplied container 
    has no more objects to return an indefinite sequence of 'sentinel' values is 
    returned. (This is almost the converse of built-in iter(c,sentinel).)
    """
    i = iter(c)
    try:
        while True: yield i.next()
    except StopIteration:
        while True: yield sentinel

def zipAll(*lists):
    """
    A zip-iterator that, unlike the built-in zip function, keeps on returning
    tuples until all elements of the supplied lists have been returned.  When
    the values from any list have been exhausted, None values are returned.
    The iterator stops when all lists have been exhausted.
    """
    iters = map(iterAll,lists)
    while True:
        result = [i.next() for i in iters]
        if allEq(None,result): break
        yield tuple(result)
    return

def isEq(v):
    """
    Return a function that tests for equality with the supplied value.
    (Curried equality function.)
    """
    return (lambda v2: v==v2)

def isNe(v):
    """
    Return a function that tests for inequality with the supplied value.
    (Curried inequality function.)
    """
    return (lambda v2: v!=v2)

def all_orig(f,c):
    """
    Do all members of c satisfy f?
    """
    for i in c: 
        if not f(i): return False
    return True

def all(p, *lsargs):
    """
    Test if all sets of members from supplied lists satisfy predicate p
    """
    return reduce(and_, map(p, *lsargs), True)

def any(p, *lsargs):
    """
    Test if all sets of members from supplied lists satisfy predicate p
    """
    return reduce(or_, map(p, *lsargs), False)

def allEq(v,c):
    """
    Are all members of c equal to v?
    """
    return all(isEq(v),c)

def allNe(v,c):
    """
    Are all members of c not equal to v?
    """
    return all(isNe(v),c)

def filterSplit(p, values):
    """
    Function filters a list into two sub-lists, the first containing entries
    satisfying the supplied predicate p, and the second of entries not satisfying p.
    """
    satp = []
    notp = []
    for v in values:
        if p(v):
            satp.append(v)
        else:
            notp.append(v)
    return (satp,notp)

def cond(cond,v1,v2):
    """
    Conditional expression.
    """
    if cond:
        return v1 
    else:
        return v2

def interleave(l1,l2):
    """
    Interleave lists.
    """
    if not l1: return l2
    if not l2: return l1
    return [l1[0],l2[0]]+interleave(l1[1:],l2[1:])

def endsWith(base,suff):
    """
    Test if list (sequence) ends with given suffix
    """
    return base[-len(suff):] == suff

def formatIntList(ints, sep=",", intfmt=str):
    """
    Format list of integers, using a supplied function to format each value,
    and inserting a supplied separator between each.
    
    Default comma-separated list of decimals.
    """
    return sep.join(map(intfmt, ints))

def formatInt(fmt):
    """
    returns a function to format a single integer value using the supplied
    format string.
    """
    def dofmt(n): return fmt % (n,)
    return dofmt

def formatList(lst,left=0,right=0):
    """
    Format a list over one or more lines, using the supplied margins.
    Left margin padding is *not* added to the first line of output,
    and no final newline is included following the last line of output.
    """
    # Try for format on one line
    out = formatList1(lst,right-left)
    if not out:
        # format over multiple lines
        out = "("
        pre = " "
        pad = "\n"+left*" "
        for i in lst:
            out += pre
            if isinstance(i,list) or isinstance(i,tuple):
                out += formatList(i, left+2, right)
            elif isinstance(i,dict):
                out += formatDict(i, left+2, right, left+2)
            else:
                out += repr(i)
            pre = pad+", "
        out += pad + ")"
    return out

def formatList1(lst,width):
    """
    Attempt to format a list on a single line, within supplied width,
    or return None if the list does not fit.
    """
    out = "("
    pre = ""
    ol  = 2
    for i in lst:
        o   = pre+repr(i)
        ol += len(o)
        if ol > width: return None
        pre  = ", "
        out += o
    return out+")"

def formatDict(dic,left=0,right=0,pos=0):
    """
    Format a dictionary over one or more lines, using the supplied margins.
    Left margin padding is *not* added to the first line of output,
    and no final newline is included following the last line of output.
    """
    # Try for format on one line
    out = formatDict1(dic,right-pos)
    if not out:
        # format over multiple lines
        out = "{"
        pre = " "
        pad = "\n"+left*" "
        for k in dic.keys():
            out += pre
            v    = dic[k]
            ks   = repr(k)+': '
            p    = pos+2+len(ks)
            if isinstance(v,dict):
                o = formatDict1(v, right-p)
                if not o:
                    o = pad + "  " + formatDict(v, left+2, right, left+2)
                out += ks + o
            elif isinstance(v,list) or isinstance(v,tuple):
                o = formatList1(v, right-p)
                if not o:
                    o = pad + "  " + formatList(v, left+2, right)
                out += ks + o
            else:
                out += ks + repr(v)
            pre  = pad+", "
            pos  = left+2
        out += pad + "}"
    return out

def formatDict1(dic,width):
    """
    Attempt to format a dictionary on a single line, within the supplied width,
    or return None if it does not fit.
    """
    out = "{"
    pre = ""
    ol  = 2
    for k in dic.keys():
        v  = dic[k]
        o  = pre + repr(k)+': '
        if isinstance(v,dict):
            vs = formatDict1(v,width)
            if not vs: return None
            o += vs
        elif isinstance(v,list) or isinstance(v,tuple):
            vs = formatList1(v,width)
            if not vs: return None
            o += vs
        else:
            o += repr(v)
        ol += len(o)
        if ol > width: return None
        pre  = ", "
        out += o
    return out+"}"

def compareLists(c1,c2):
    """
    Compare a pair of lists, returning None if the lists are identical, 
    or a pair of lists containing:
    (1) elements of first list not in second, and
    (2) elements of second list not in first list.
    """
    c1 = c1 or []
    c2 = c2 or []
    c1d = []
    c2d = []
    for c in c1: 
        if not (c in c2): c1d.append(c)
    for c in c2: 
        if not (c in c1): c2d.append(c)
    if c1d or c2d: return (c1d,c2d)
    return None

def compareDicts(d1,d2):
    """
    Return None if dictionaries are identical, or pair of lists containing
    entries in d1 not in d2, and entries in d2 not in d1.
    """
    dif1 = diffDicts(d1,d2)
    dif2 = diffDicts(d2,d1)
    if dif1 or dif2:
        return (dif1,dif2)
    else:
        return None

def diffDicts(d1,d2):
    """
    Return dictionary of entries in d1 that are not in d2.
    """
    difs = {}
    for (k,v1) in d1.iteritems():
        if v1:
            if k not in d2:
                difs[k] = v1
            else:
                d = diffPair(v1,d2[k])
                if nonEmpty(d):
                    difs[k] = d
    return difs

def diffLists(t1,t2):
    """
    Compares pairwise elements of 2 lists, and returns a list of elements
    in the first that are not in the second.
    Where the elements are dictionaries or tuples, the element difference is
    determined recursively, otherwise the value is treated atomically.
    """
    ps = zipAll(t1,t2)
    ds = filter(nonEmpty, [diffPair(a,b) for (a,b) in ps])
    return ds

def diffTuples(t1,t2):
    """
    Compares pairwise elements of 2 tuples, and returns a list of elements
    in the first that are not in the second.
    Where the elements are dictionaries or tuples, the element difference is
    determined recursively, otherwise the value is treated atomically.
    """
    return tuple(diffLists(t1,t2))
    
def diffPair(v1,v2):
    """
    Return the part of v1 that is not present in v2.
    Returns None if v11 and v2 are equal, or if every element of v1 is
    also present in v2.
    """   
    if isinstance(v1,tuple) and isinstance(v2,tuple):
        return diffTuples(v1,v2)
    if isinstance(v1,list) and isinstance(v2,list):
        return diffLists(v1,v2)
    if isinstance(v1,dict) and isinstance(v2,dict):
        return diffDicts(v1,v2)
    if v1!=v2:
        return v1
    return None

def nonEmpty(v):
    """
    If v is a container (tuple, list or dictionary), return None if it is empty,
    otherwise return v itself.
    """
    if isinstance(v,(tuple,list,dict)):
        if len(v) == 0: return None
    return v

# End.



# $Id: TestTestUtils.py 1047 2009-01-15 14:48:58Z graham $
#
# Test test runner utiltities
#

import sys
import logging
import unittest

sys.path.append("../..")

# Test class
class TestExample(unittest.TestCase):

    def setUp(self):
        return

    def tearDown(self):
        return

    # Test cases

    def testCase(self):
        assert "Some condition"

    # Sentinel/placeholder tests

    def testUnits(self):
        assert (True)

    def testComponents(self):
        assert (True)

    def testIntegration(self):
        assert (True)

    def testPending(self):
        assert (False), "No pending test"

# Assemble test suite

from MiscLib import TestUtils

def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit": 
            [ "testUnits"
            , "testCase"
            ],
        "component":
            [ "testComponents"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            ]
        }
    return TestUtils.getTestSuite(TestExample, testdict, select=select)

# Run unit tests directly from command line
if __name__ == "__main__":
    TestUtils.runTests("TestExample", getTestSuite, sys.argv)

# End.



# $Id: TestSuperGlobal.py 1047 2009-01-15 14:48:58Z graham $
#
# Unit testing for SuperGlobal module
# See http://pyunit.sourceforge.net/pyunit.html
#

import sys
import unittest

sys.path.append("../..")
from MiscLib.SuperGlobal import *

class TestSuperGlobal(unittest.TestCase):

    def setUp(self):
        return

    def tearDown(self):
        return

    # Test cases

    def testCreateSuperGlobal(self):
        sg = SuperGlobal()
        sg.i1 = 1
        sg.s1 = "s1"
        self.assertEqual(sg.i1, 1)
        self.assertEqual(sg.s1, "s1")

    def testRetrieveSuperGlobal(self):
        self.testCreateSuperGlobal()
        sg = SuperGlobal()
        self.assertEqual(sg.i1, 1)
        self.assertEqual(sg.s1, "s1")

    def testUpdateSuperGlobal(self):
        self.testCreateSuperGlobal()
        sg = SuperGlobal()
        sg.i2 = 2
        sg.s2 = "s2"
        self.assertEqual(sg.i1, 1)
        self.assertEqual(sg.s1, "s1")
        self.assertEqual(sg.i2, 2)
        self.assertEqual(sg.s2, "s2")
        sg2 = SuperGlobal()
        sg2.i1 = 11
        sg2.s1 = "s11"
        self.assertEqual(sg.i1, 11)
        self.assertEqual(sg.s1, "s11")
        self.assertEqual(sg.i2, 2)
        self.assertEqual(sg.s2, "s2")

# Code to run unit tests directly from command line.
def getTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(TestSuperGlobal("testCreateSuperGlobal"))
    suite.addTest(TestSuperGlobal("testRetrieveSuperGlobal"))
    suite.addTest(TestSuperGlobal("testUpdateSuperGlobal"))
    return suite

if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(getTestSuite())

# End.



# $Id: TestDomHelpers.py 1058 2009-01-26 10:39:19Z graham $
#
# Unit testing for WebBrick library functions (Functions.py)
# See http://pyunit.sourceforge.net/pyunit.html
#

import sys
import unittest
import logging
from os import *
from os.path import *

sys.path.append("../..")
from MiscLib.DomHelpers import *

class TestDomHelpers(unittest.TestCase):

    def setUp(self):
        self.testdoc = ( """<?xml version="1.0" encoding="iso-8859-1" ?>
<root attr="attrtext">
  <child1>
  some text
  <child11 />
  more text
  <child12>child text</child12>
  final text
  </child1>
</root>""" )
        self.testpath = "resources/"
        self.testfile = self.testpath+"TestDomHelpers.xml"
        self.savefile = self.testpath+"TestDomHelpersSave.xml"
        return

    def tearDown(self):
        return

    def doAssert(self, cond, msg):
        assert cond , msg

    # Actual tests follow

    def testParseXmlString(self):
        assert parseXmlString(self.testdoc), "Parse XML string failed"

    def testParseXmlFile(self):
        assert parseXmlFile(self.testfile), "Parse XML file failed"

    def testSaveXmlToFile(self):
        # ensure clean first
        logging.debug( self.testfile )
        logging.debug( self.savefile )
        try:
            remove( self.savefile )
        except Exception:
            pass
        testDom = parseXmlFile(self.testfile)
        saveXmlToFile(self.savefile, testDom, False)
        
        assert exists(self.savefile), "save XML file failed"
        # could expand test and run compare of data.
#        remove( self.savefile )

    def testSaveXmlToFileWithBackup(self):
        # ensure clean first
        try:
            remove( self.savefile )
        except Exception:
            pass
        testDom = parseXmlFile(self.testfile)
        saveXmlToFile(self.savefile, testDom)
        saveXmlToFile(self.savefile, testDom)
        
        assert exists(self.savefile), "save XML file failed"
        assert exists(self.savefile+".bak"), "save XML file failed"

        remove( self.savefile+".bak" )
        remove( self.savefile )

    def testGetNamedElem1(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "child1")
        assert elm, "Node not found: child1"

    def testGetNamedElem2(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "child12")
        assert elm, "Node not found: child12"

    def testGetNamedElem3(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "child99")
        assert not elm, "Unexpected dode found: child99"

    #TODO: deprecate me
    def testGetNamedNode(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedNode(dom, "child1")
        assert elm, "Node not found: child1"

    def testElemText(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "child1")
        txt = getElemText(elm)
        assert txt == "\n  some text\n  \n  more text\n  \n  final text\n  ", \
                      "Wrong element text: "+repr(txt)

    def testAttrText(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        txt = getAttrText(elm,"attr")
        assert txt == "attrtext", "Wrong attribute text: "+txt

    def testNamedNodeAttrText(self):
        dom = parseXmlString(self.testdoc)
        txt = getNamedNodeAttrText(dom, "root", "attr")
        self.assertEqual( txt, "attrtext" )

    def testNodeListText(self):
        dom = parseXmlFile(self.testfile)
        elm = dom.getElementsByTagName("child1")[0]
        txt = getNodeListText(elm.childNodes)
        assert txt == "\n    some text\n    \n    more text\n    \n    final text\n  ", \
                      "Wrong element text: "+repr(txt)

    def testGetNamedNodeText(self):
        dom = parseXmlFile(self.testfile)
        txt = getNamedNodeText(dom, "child1")
        self.assertEqual( txt, "\n    some text\n    \n    more text\n    \n    final text\n  " )

    def testgetElemXml(self):
        dom = parseXmlFile(self.testfile)
        txt = getElemXml( dom.getElementsByTagName("child1")[0] )
        self.assertEqual( txt, "<child1>\n    some text\n    <child11/>\n    more text\n    <child12>child text</child12>\n    final text\n  </child1>" )

    def testgetElemPrettyXml(self):
        dom = parseXmlFile(self.testfile)
        txt = getElemPrettyXml( dom.getElementsByTagName("child1")[0] )
        self.assertEqual( txt, "<child1>\n  \n    some text\n    \n  <child11/>\n  \n    more text\n    \n  <child12>\n    child text\n  </child12>\n  \n    final text\n  \n</child1>\n" )

    def testGetNamedNodeXml(self):
        dom = parseXmlFile(self.testfile)
        txt = getNamedNodeXml( dom, "child1" )
        self.assertEqual( txt, "<child1>\n    some text\n    <child11/>\n    more text\n    <child12>child text</child12>\n    final text\n  </child1>" )

    # --- node type tests ---

    def testIsAttribute(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        atr = elm.getAttributeNode("attr")
        assert isAttribute(atr),"isAttribute test (root/@attr) failed"

    def testIsAttributeElem(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        assert not isAttribute(elm),"isAttribute false test (root) failed"

    def testIsAttributeText(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        txt = elm.childNodes[0]
        assert not isAttribute(elm),"isAttribute false test (root/[0]) failed"

    def testIsElement(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        assert isElement(elm),"isElement test (root/@attr) failed"

    def testIsElementAttr(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        atr = elm.getAttributeNode("attr")
        assert not isElement(atr),"isElement false test (root/@attr) failed"

    def testIsElementText(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        txt = elm.childNodes[0]
        assert not isElement(txt),"isElement false test (root/[0]) failed"

    def testIsText(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        txt = elm.childNodes[0]
        assert isText(txt),"isText test (root/[0]) failed"

    def testIsTextAttr(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        atr = elm.getAttributeNode("attr")
        assert not isText(atr),"isText false test (root/@attr) failed"

    def testIsTextElem(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        assert not isText(elm),"isText false test (root) failed"

    def testRemoveChildren(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        el1 = removeChildren(elm)
        self.assertEqual(getElemXml(elm),"""<root attr="attrtext"/>""")
        self.assertEqual(elm.childNodes, [])

    def testReplaceChildren(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        c1  = getNamedElem(elm, "child11")
        c2  = getNamedElem(elm, "child12")
        replaceChildren(elm,[c1,c2])
        self.assertEqual(getElemXml(elm),
          """<root attr="attrtext">"""+
          """<child11/>"""+
          """<child12>child text</child12>"""+
          """</root>""")

    def testReplaceChildrenText(self):
        dom = parseXmlString(self.testdoc)
        elm = getNamedElem(dom, "root")
        replaceChildrenText(elm,"replacement text")
        self.assertEqual(getElemXml(elm),
          """<root attr="attrtext">"""+
          """replacement text"""+
          """</root>""")

    def testEscapeText(self):
        self.assertEqual(
          escapeText("<tag>Jack & Jill</tag>"),
          "&lt;tag&gt;Jack &amp; Jill&lt;/tag&gt;")

    def testEscapeTextForHtml(self):
        self.assertEqual(
          escapeTextForHtml("<tag>\nJack & Jill\n</tag>\n"),
          "&lt;tag&gt;<br/>Jack &amp; Jill<br/>&lt;/tag&gt;<br/>")

    def testPrintDictionary(self, dic):
        for key in dic:
            if isinstance( dic[key], list ):
                logging.debug( "%s : list " % ( key ) )
                for ntry in dic[key]:
                    # print list
                    if isinstance( ntry, dict ):
                        self.testPrintDictionary(ntry )
                    else:
                        logging.debug( "%s : %s " % ( key, ntry ) )
            elif isinstance( dic[key], dict ):
                logging.debug( "%s : dictionary " % ( key ) )
                self.testPrintDictionary(dic[key] )
            else:
                logging.debug( "%s : %s " % ( key, dic[key] ) )

    def testGetXmlFromDict(self):
        testDict = { 'root': { '': 'string', 
                            'child1': { '': 'string1', 'attr1':'attr1text' },
                            'child2': { '': 'string2', 'attr2':'attr2text',
                                    'child21': { '': 'string21', 'attr21':'attr21text' }, 
                                    },
                            } 
                   }
        testStr = ("""<?xml version="1.0" encoding="utf-8"?>"""+
                   """<root>string<child1 attr1="attr1text">string1</child1>"""+
                   """<child2 attr2="attr2text">string2"""+
                   """<child21 attr21="attr21text">string21</child21>"""+
                   """</child2></root>""")

        dom = getXmlDomFromDict( testDict )

        domTxt = getElemXml( dom )

        logging.debug( repr(testDict) )
        logging.debug( repr(dom) )
        logging.debug( domTxt )
        logging.debug( testStr )

        testDict = getDictFromXml( dom )
        logging.debug( repr(testDict) )

        #add some checks
        #order is not always the same as the dictionary has its own random order
        self.assertEqual( domTxt, testStr )

    def testGetXmlFromDict2(self):
        testDict = { 'root': { 'simpleVal': 'string', 
                            'listVal':[ 'string1', 'string2', 'string3' ],
                            'listVal2':[ { 'key11': 'string11', 'key12': 'string12', 'key13': 'string13'}, 
                                        { 'key21': 'string21', 'key22': 'string22', 'key23': 'string23'},
                                        ] 
                            } 
                   }
        testStr = """<?xml version="1.0" encoding="utf-8"?><root><simpleVal>string</simpleVal><listVal>string1</listVal><listVal>string2</listVal><listVal>string3</listVal><listVal2><key11>string11</key11><key12>string12</key12><key13>string13'</key13><listVal2><listVal2><key21>string21</key21><key22>string22</key22><key23>string23'</key23><listVal2><root>"""

        dom = getXmlDomFromDict( testDict )

        domTxt = getElemXml( dom )

        logging.debug( repr(testDict) )
        logging.debug( repr(dom) )
        logging.debug( domTxt )
        logging.debug( testStr )

        testDict = getDictFromXml( dom )
        logging.debug( repr(testDict) )

        #add some checks
        #order is not always the same as the dictionary has its own random order
        #self.assertEqual( domTxt, testStr )

    def testGetDictFromSimpleXml(self):

        self.testdoc = ( """<?xml version="1.0" encoding="iso-8859-1" ?>
<root attr="attrtext">
  <child1>child1 text</child1>
  <child2 attr2="attr2text">child2 text</child2>
</root>""" )

        logging.debug( self.testdoc )
        dom = parseXmlString(self.testdoc)
        testDict = getDictFromXml( dom )
        logging.debug( repr(testDict) )
        #self.testPrintDictionary(testDict)
        self.assert_( testDict.has_key( "root" ) )

        rootDict = testDict["root"]
        self.assert_( rootDict.has_key( "attr" ) )
        self.assert_( isinstance(rootDict["attr"], basestring) )
        self.assertEqual( rootDict["attr"], "attrtext" )

        self.assert_( rootDict.has_key( "child1" ) )
        self.assert_( isinstance(rootDict["child1"], dict) )
        child1Dict = rootDict["child1"]
        self.assert_( child1Dict.has_key( "" ) )    # text element
        self.assertEqual( child1Dict[""], "child1 text" )

        self.assert_( rootDict.has_key( "child2" ) )
        self.assert_( isinstance(rootDict["child2"], dict) )
        child2Dict = rootDict["child2"]
        self.assert_( child2Dict.has_key( "" ) )    # text element
        self.assertEqual( child2Dict[""], "child2 text" )
        self.assert_( child2Dict.has_key( "attr2" ) )    # text element
        self.assert_( isinstance(child2Dict["attr2"], basestring) )
        self.assertEqual( child2Dict["attr2"], "attr2text" )
                      
    def testGetDictFromXml(self):

        self.testdoc = ( """<?xml version="1.0" encoding="iso-8859-1" ?>
<root attr="attrtext">
  <child1>
  some text
  <child11 />
  more text
  <child12>child text</child12>
  final text
  </child1>
  <repeats>
      <repeat>
        repeat 1
      </repeat>
      <repeat>
        repeat 2
      </repeat>
  </repeats>
  <repeat>
    repeat 1
  </repeat>
  <repeat>
    repeat 2
  </repeat>
</root>""" )

        logging.debug( repr(self.testdoc) )
        dom = parseXmlString(self.testdoc)
        testDict = getDictFromXml( dom )
        logging.debug( repr(testDict) )
        self.assert_( testDict.has_key( "root" ) )
        rootDict = testDict["root"]
        self.assert_( rootDict.has_key( "child1" ) )
        child1Dict = rootDict["child1"]
        self.assertEqual( child1Dict[""],
                    "some text\n  \n  more text\n  \n  final text" )
        self.assertEqual( rootDict["attr"],
                    "attrtext" )
        self.assert_( child1Dict.has_key( "child11" ) )
        self.assert_( child1Dict.has_key( "child12" ) )
        self.assertEqual( child1Dict["child12"][''],
                    "child text" )
        self.assert_( rootDict.has_key( "repeats" ) )
        self.assert_( isinstance( rootDict["repeats"], list ) )
        self.assert_( rootDict.has_key( "repeat" ) )
        self.assert_( isinstance( rootDict["repeat"], list ) )

    def testGetDictFromXml2(self):

        self.testdoc = ( """<?xml version="1.0" encoding="iso-8859-1" ?>
<eventInterfaces>
    <eventInterface module='TestDespatchTask' name='TestEventLogger'>
        <!-- This saves all events -->
        <eventtype type="type1">
            <eventsource source="source1" >
            <event>
                    <!-- interested in all events -->
            </event>
            </eventsource>
        </eventtype>
    </eventInterface>
    <eventInterface module='TestDespatchTask2' name='TestEventLogger2'>
        <!-- This saves all events -->
        <eventtype type="type2">
            <eventsource source="source2" >
            <event>
                    <!-- interested in all events -->
            </event>
            </eventsource>
            <eventsource source="source3" >
            <event>
                    <!-- interested in all events -->
            </event>
            </eventsource>
        </eventtype>
        <eventtype type="type3">
            <eventsource source="source4" >
            <event>
                    <!-- interested in all events -->
                    <params>
                        <second type="list">5,20,35,50</second>
                    </params>
            </event>
            <event>
                    <!-- interested in all events -->
                    <params>
                        <second>5</second>
                    </params>
            </event>
            </eventsource>
        </eventtype>
    </eventInterface>
</eventInterfaces>""" )

        logging.debug( repr(self.testdoc) )
        dom = parseXmlString(self.testdoc)
        testDict = getDictFromXml( dom )
        logging.debug( repr(testDict) )
        self.assert_( testDict.has_key( "eventInterfaces" ) )
        self.testPrintDictionary( testDict )

# Code to run unit tests directly from command line.
# Constructing the suite manually allows control over the order of tests.
def getTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(TestDomHelpers("testParseXmlString"))
    suite.addTest(TestDomHelpers("testParseXmlFile"))
    suite.addTest(TestDomHelpers("testSaveXmlToFile"))
    suite.addTest(TestDomHelpers("testSaveXmlToFileWithBackup"))
    suite.addTest(TestDomHelpers("testGetNamedElem1"))
    suite.addTest(TestDomHelpers("testGetNamedElem2"))
    suite.addTest(TestDomHelpers("testGetNamedElem3"))

    suite.addTest(TestDomHelpers("testElemText"))
    suite.addTest(TestDomHelpers("testAttrText"))
    suite.addTest(TestDomHelpers("testNodeListText"))
    suite.addTest(TestDomHelpers("testIsAttribute"))
    suite.addTest(TestDomHelpers("testIsAttributeElem"))
    suite.addTest(TestDomHelpers("testIsAttributeText"))
    suite.addTest(TestDomHelpers("testIsElement"))
    suite.addTest(TestDomHelpers("testIsElementAttr"))
    suite.addTest(TestDomHelpers("testIsElementText"))
    suite.addTest(TestDomHelpers("testIsText"))
    suite.addTest(TestDomHelpers("testIsTextAttr"))
    suite.addTest(TestDomHelpers("testIsTextElem"))

    suite.addTest(TestDomHelpers("testGetNamedNodeXml"))
    suite.addTest(TestDomHelpers("testgetElemXml"))
    suite.addTest(TestDomHelpers("testgetElemPrettyXml"))
    suite.addTest(TestDomHelpers("testGetNamedNodeText"))
    suite.addTest(TestDomHelpers("testNamedNodeAttrText"))

    suite.addTest(TestDomHelpers("testRemoveChildren"))
    suite.addTest(TestDomHelpers("testReplaceChildren"))
    suite.addTest(TestDomHelpers("testReplaceChildrenText"))

    suite.addTest(TestDomHelpers("testEscapeText"))
    suite.addTest(TestDomHelpers("testEscapeTextForHtml"))
    
    return suite

if __name__ == "__main__":
    # unittest.main()

    if len(sys.argv) > 1:
        logging.basicConfig(level=logging.DEBUG)
        tests = TestDomHelpers( sys.argv[1] )
    else:
        tests = getTestSuite()

    runner = unittest.TextTestRunner(verbosity=2)
    runner.run(tests)



# $Id: TestScanDirectories.py 1058 2009-01-26 10:39:19Z graham $
#
# Unit testing for ScanDirectory functions
# See http://pyunit.sourceforge.net/pyunit.html
#

import sys
import unittest
import re
import logging
from os.path import normpath, abspath

sys.path.append("../..")
from MiscLib.ScanDirectories import CollectDirectoryContents
from MiscLib.Functions import compareLists
from MiscLib import TestUtils

class TestScanDirectories (unittest.TestCase):

    def setUp(self):
        self.srcPath = abspath("./resources/")
        self.basePath = abspath(".") #+"/"
        return

    def tearDown(self):
        return

    # Actual tests follow

    def testCollectDirsShallow(self):
        dirs     = CollectDirectoryContents(self.srcPath, baseDir=self.basePath, recursive=False)
        expected = [ "resources/TestScanDir1"
                   , "resources/TestScanDir2"
                   , "resources/TestScanFilesSubDir"
                   ]
        c = compareLists(dirs, expected)
        assert c == None, "Wrong directory list: "+repr(c)

    def testCollectDirsRecursive(self):
        dirs     = CollectDirectoryContents(self.srcPath,self.basePath)
        expected = [ "resources/TestScanDir1"
                   , "resources/TestScanDir1/SubDir1a"
                   , "resources/TestScanDir1/SubDir1b"
                   , "resources/TestScanDir2"
                   , "resources/TestScanDir2/SubDir2"
                   , "resources/TestScanFilesSubDir"
                   ]
        c = compareLists(dirs, expected)
        assert c == None, "Wrong directory list: "+repr(c)

    def testCollectDirsRecursiveBaseEndswithSep(self):
        dirs     = CollectDirectoryContents(self.srcPath,self.basePath+"/")
        expected = [ "resources/TestScanDir1"
                   , "resources/TestScanDir1/SubDir1a"
                   , "resources/TestScanDir1/SubDir1b"
                   , "resources/TestScanDir2"
                   , "resources/TestScanDir2/SubDir2"
                   , "resources/TestScanFilesSubDir"
                   ]
        c = compareLists(dirs, expected)
        assert c == None, "Wrong directory list: "+repr(c)

    def testCollectDirsRecursiveEmptyBase(self):
        dirs     = CollectDirectoryContents(self.srcPath, baseDir="")
        expected = [ self.basePath+"/resources/TestScanDir1"
                   , self.basePath+"/resources/TestScanDir1/SubDir1a"
                   , self.basePath+"/resources/TestScanDir1/SubDir1b"
                   , self.basePath+"/resources/TestScanDir2"
                   , self.basePath+"/resources/TestScanDir2/SubDir2"
                   , self.basePath+"/resources/TestScanFilesSubDir"
                   ]
        c = compareLists(dirs, expected)
        assert c == None, "Wrong directory list: "+repr(c)

    def testCollectFilesShallow(self):
        dirs     = CollectDirectoryContents(self.srcPath, baseDir=self.basePath, 
                        listDirs=False, listFiles=True, recursive=False)
        expected = [ "resources/TestDomHelpers.xml"
                   , "resources/TestScanFiles1.txt"
                   , "resources/TestScanFiles2.txt"
                   ]
        c = compareLists(dirs, expected)
        assert c == None, "Wrong directory list: "+repr(c)

    def testCollectFilesRecursive(self):
        dirs     = CollectDirectoryContents(self.srcPath, baseDir=self.basePath,
                        listDirs=False, listFiles=True, recursive=True)
        expected = [ "resources/TestDomHelpers.xml"
                   , "resources/TestScanFiles1.txt"
                   , "resources/TestScanFiles2.txt"
                   , "resources/TestScanFilesSubDir/TestScanFiles31.txt"
                   , "resources/TestScanFilesSubDir/TestScanFiles32.txt"
                   ]
        c = compareLists(dirs, expected)
        assert c == None, "Wrong directory list: "+repr(c)

    def testCollectFilesRecursiveBaseEndswithSep(self):
        dirs     = CollectDirectoryContents(self.srcPath, baseDir=self.basePath+"/",
                        listDirs=False, listFiles=True, recursive=True)
        expected = [ "resources/TestDomHelpers.xml"
                   , "resources/TestScanFiles1.txt"
                   , "resources/TestScanFiles2.txt"
                   , "resources/TestScanFilesSubDir/TestScanFiles31.txt"
                   , "resources/TestScanFilesSubDir/TestScanFiles32.txt"
                   ]
        c = compareLists(dirs, expected)
        assert c == None, "Wrong directory list: "+repr(c)

    def testCollectFilesRecursiveEmptyBase(self):
        dirs     = CollectDirectoryContents(self.srcPath, baseDir="",
                        listDirs=False, listFiles=True, recursive=True)
        expected = [ self.basePath+"/resources/TestDomHelpers.xml"
                   , self.basePath+"/resources/TestScanFiles1.txt"
                   , self.basePath+"/resources/TestScanFiles2.txt"
                   , self.basePath+"/resources/TestScanFilesSubDir/TestScanFiles31.txt"
                   , self.basePath+"/resources/TestScanFilesSubDir/TestScanFiles32.txt"
                   ]
        c = compareLists(dirs, expected)
        assert c == None, "Wrong directory list: "+repr(c)

    def testCollectAllShallow(self):
        dirs     = CollectDirectoryContents(self.srcPath, baseDir=self.basePath, 
                        listDirs=True, listFiles=True, recursive=False)
        expected = [ "resources/TestScanDir1"
                   , "resources/TestScanDir2"
                   , "resources/TestScanFilesSubDir"
                   , "resources/TestDomHelpers.xml"
                   , "resources/TestScanFiles1.txt"
                   , "resources/TestScanFiles2.txt"
                   ]
        c = compareLists(dirs, expected)
        assert c == None, "Wrong directory list: "+repr(c)

    def testCollectAllRecursive(self):
        dirs     = CollectDirectoryContents(self.srcPath, baseDir=self.basePath,
                        listDirs=True, listFiles=True, recursive=True)
        expected = [ "resources/TestScanDir1"
                   , "resources/TestScanDir1/SubDir1a"
                   , "resources/TestScanDir1/SubDir1b"
                   , "resources/TestScanDir2"
                   , "resources/TestScanDir2/SubDir2"
                   , "resources/TestScanFilesSubDir"
                   , "resources/TestDomHelpers.xml"
                   , "resources/TestScanFiles1.txt"
                   , "resources/TestScanFiles2.txt"
                   , "resources/TestScanFilesSubDir/TestScanFiles31.txt"
                   , "resources/TestScanFilesSubDir/TestScanFiles32.txt"
                   ]
        c = compareLists(dirs, expected)
        assert c == None, "Wrong directory list: "+repr(c)

    def testCollectAllRecursiveBaseEndswithSep(self):
        dirs     = CollectDirectoryContents(self.srcPath, baseDir=self.basePath+"/",
                        listDirs=True, listFiles=True, recursive=True)
        expected = [ "resources/TestScanDir1"
                   , "resources/TestScanDir1/SubDir1a"
                   , "resources/TestScanDir1/SubDir1b"
                   , "resources/TestScanDir2"
                   , "resources/TestScanDir2/SubDir2"
                   , "resources/TestScanFilesSubDir"
                   , "resources/TestDomHelpers.xml"
                   , "resources/TestScanFiles1.txt"
                   , "resources/TestScanFiles2.txt"
                   , "resources/TestScanFilesSubDir/TestScanFiles31.txt"
                   , "resources/TestScanFilesSubDir/TestScanFiles32.txt"
                   ]
        c = compareLists(dirs, expected)
        assert c == None, "Wrong directory list: "+repr(c)

    def testCollectAllRecursiveEmptyBase(self):
        dirs     = CollectDirectoryContents(self.srcPath, baseDir="",
                        listDirs=True, listFiles=True, recursive=True)
        expected = [ self.basePath+"/resources/TestScanDir1"
                   , self.basePath+"/resources/TestScanDir1/SubDir1a"
                   , self.basePath+"/resources/TestScanDir1/SubDir1b"
                   , self.basePath+"/resources/TestScanDir2"
                   , self.basePath+"/resources/TestScanDir2/SubDir2"
                   , self.basePath+"/resources/TestScanFilesSubDir"
                   , self.basePath+"/resources/TestDomHelpers.xml"
                   , self.basePath+"/resources/TestScanFiles1.txt"
                   , self.basePath+"/resources/TestScanFiles2.txt"
                   , self.basePath+"/resources/TestScanFilesSubDir/TestScanFiles31.txt"
                   , self.basePath+"/resources/TestScanFilesSubDir/TestScanFiles32.txt"
                   ]
        c = compareLists(dirs, expected)
        assert c == None, "Wrong directory list: "+repr(c)

    # Sentinel/placeholder tests

    def testUnits(self):
        assert (True)

    def testComponents(self):
        assert (True)

    def testIntegration(self):
        assert (True)

    def testPending(self):
        assert (False), "Pending tests follow"

# Assemble test suite

def getTestSuite(select="unit"):
    """
    Get test suite

    select  is one of the following:
            "unit"      return suite of unit tests only
            "component" return suite of unit and component tests
            "all"       return suite of unit, component and integration tests
            "pending"   return suite of pending tests
            name        a single named test to be run
    """
    testdict = {
        "unit":
            [ "testUnits"
            , "testCollectDirsShallow"
            , "testCollectDirsRecursive"
            , "testCollectDirsRecursiveBaseEndswithSep"
            , "testCollectDirsRecursiveEmptyBase"
            , "testCollectFilesShallow"
            , "testCollectFilesRecursive"
            , "testCollectFilesRecursiveBaseEndswithSep"
            , "testCollectFilesRecursiveEmptyBase"
            , "testCollectAllShallow"
            , "testCollectAllRecursive"
            , "testCollectAllRecursiveBaseEndswithSep"
            , "testCollectAllRecursiveEmptyBase"
            ],
        "component":
            [ "testComponents"
            ],
        "integration":
            [ "testIntegration"
            ],
        "pending":
            [ "testPending"
            ]
        }
    return TestUtils.getTestSuite(TestScanDirectories, testdict, select=select)

if __name__ == "__main__":
    TestUtils.runTests("TestScanDirectories.log", getTestSuite, sys.argv)
    #logging.basicConfig(level=logging.DEBUG)
    #runner = unittest.TextTestRunner()
    #runner.run(getTestSuite())
    


#!/usr/bin/env python

"""
ASQC - A SPARQL query client
"""

import sys
import os
import os.path
import urlparse
import urllib
import urllib2
import StringIO
import json
import re
import optparse
import logging

from SparqlHttpClient import SparqlHttpClient

from StdoutContext import SwitchStdout
from StdinContext  import SwitchStdin

import rdflib

# Set up to use SPARQL
rdflib.plugin.register(
    'sparql', rdflib.query.Processor,
    'rdfextras.sparql.processor', 'Processor')
rdflib.plugin.register(
    'sparql', rdflib.query.Result,
    'rdfextras.sparql.query', 'SPARQLQueryResult')

# Logging object
log = logging.getLogger(__name__)

class asqc_settings(object):
    VERSION = "v0.1"

# Helper functions for JSON formatting and parsing
# Mostly copied from rdflib SPARQL code (rdfextras/sparql/results/jsonresults)

def termToJSON(term): 
    if isinstance(term, rdflib.URIRef): 
        return { 'type': 'uri', 'value': str(term) }
    elif isinstance(term, rdflib.Literal):
        if term.datatype!=None:
            return { 'type': 'typed-literal', 
                     'value': unicode(term), 
                     'datatype': str(term.datatype) }
        else:
            r={'type': 'literal',
               'value': unicode(term) }
            if term.language!=None:
                r['xml:lang']=term.language
            return r
    elif isinstance(term, rdflib.BNode):
        return { 'type': 'bnode', 'value': str(term) }
    elif term==None: 
        return None
    else: 
        raise rdflib.query.ResultException('Unknown term type: %s (%s)'%(term, type(term)))

def bindingToJSON(binding):
    res={}
    for var in binding: 
        t = termToJSON(binding[var])
        if t != None: res[var] = t
    return res

def parseJsonTerm(d):
    """rdflib object (Literal, URIRef, BNode) for the given json-format dict.
    
    input is like:
      { 'type': 'uri', 'value': 'http://famegame.com/2006/01/username' }
      { 'type': 'literal', 'value': 'drewp' }
    """
    
    t = d['type']
    if t == 'uri':
        return rdflib.URIRef(d['value'])
    elif t == 'literal':
        if 'xml:lang' in d: 
            return rdflib.Literal(d['value'], lang=d['xml:lang'])
        return rdflib.Literal(d['value'])
    elif t == 'typed-literal':
        return rdflib.Literal(d['value'], datatype=rdflib.URIRef(d['datatype']))
    elif t == 'bnode': 
        return rdflib.BNode(d['value'])
    else:
        raise NotImplementedError("json term type %r" % t)

def parseJsonBindings(bindings):
    newbindings = []
    for row in bindings:
        outRow = {}
        for k, v in row.items():
            outRow[k] = parseJsonTerm(v)
        newbindings.append(outRow)
    return newbindings

# Helper functions to form join(?) of mutiple binding sets

def joinBinding(result_binding, constraint_binding):
    for k in result_binding:
        if k in constraint_binding:
            if result_binding[k] != constraint_binding[k]:
                return None
    joined_binding = result_binding.copy()
    joined_binding.update(constraint_binding)
    return joined_binding

def joinBindings(result_bindings, constraint_bindings):
    return [ bj 
             for bj in [ joinBinding(b1, b2) for b1 in result_bindings for b2 in constraint_bindings ]
             if bj ]

def joinBindingsToJSON(result_bindings, constraint_bindings):
    return [ bindingToJSON(bj) 
             for bj in [ joinBinding(b1, b2) for b1 in result_bindings for b2 in constraint_bindings ]
             if bj ]

# Helper functions for accessing data at URI reference, which may be a path relative to current directory

def resolveUri(uriref, base, path=""):
    """
    Resolve a URI reference against a supplied basae URI and path.
    (The path is a local file system path, and may need convertting to use URI conventions)
    """
    upath = urllib.pathname2url(path)
    if os.path.isdir(path) and not upath.endswith('/'):
        upath = upath + '/'
    return urlparse.urljoin(urlparse.urljoin(base, upath), uriref)

def retrieveUri(uriref):
    uri = resolveUri(uriref, "file://", os.getcwd())
    request  = urllib2.Request(uri)
    try:
        response = urllib2.urlopen(request)
        result   = response.read()
    except:
        result = None
    return result

# Helper function for determining type of query

def queryType(query):
    """
    Returns "ASK", "SELECT", "CONSTRUCT", "DESCRIBE" or None
    """
    iriregex    = "<[^>]*>"
    baseregex   = ".*base.*"+iriregex
    prefixregex = ".*prefix.*"+iriregex
    queryregex  = "^("+baseregex+")?("+prefixregex+")*.*(ask|select|construct|describe).*$"
    match = re.match(queryregex, query, flags=re.IGNORECASE|re.DOTALL)
    if match:
        return match.group(3).upper()
    return None

# Main program functions

def getQuery(options, args):
    """
    Get query string from command line option or argument.
    """
    if options.query:
        return retrieveUri(options.query)
    elif len(args) >= 2:
        return args[1]
    return None

def getPrefixes(options):
    """
    Get prefix string from command line option.
    """
    defaultPrefixes = """
        PREFIX rdf:        <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
        PREFIX rdfs:       <http://www.w3.org/2000/01/rdf-schema#>
        PREFIX owl:        <http://www.w3.org/2002/07/owl#>
        PREFIX xsd:        <http://www.w3.org/2001/XMLSchema#>
        PREFIX dcterms:    <http://purl.org/dc/terms/>
        PREFIX foaf:       <http://xmlns.com/foaf/0.1/>
        """
    #    PREFIX xml:        <http://www.w3.org/XML/1998/namespace>
    configbase = os.path.expanduser("~")
    prefixUri  = options.prefix or resolveUri(".asqc-prefixes", "file://", configbase)
    prefixes   = retrieveUri(prefixUri)
    return prefixes or defaultPrefixes

def getBindings(options):
    bndtext  = None
    bindings = (
        { "head":    { "vars": [] }
        , "results": { "bindings": [{}] }
        })
    if options.bindings and options.bindings != "-":
        bndtext = retrieveUri(options.bindings)
    elif options.bindings == "-":
        if options.rdf_data or options.endpoint:
            bndtext = sys.stdin.read()
        else:
            # Can't read bindings from stdin if trying to read RDF from stdin
            return None
    else:
        bndtext = None
    if bndtext:
        try:
            bindings = json.loads(bndtext)
            bindings['results']['bindings'] = parseJsonBindings(bindings['results']['bindings'])
        except Exception as e:
            bindings = None
    return bindings

def getRdfData(options):
    """
    Reads RDF data from files specified using -r or from stdin
    """
    if not options.rdf_data:
        options.rdf_data = ['-']
    rdfgraph = rdflib.Graph()
    for r in options.rdf_data:
        if r == "-":
            rdftext = sys.stdin.read()
        else:
            rdftext = retrieveUri(r)
        try:
            rdfgraph.parse(data=rdftext)
        except Exception, e:
            return None
    return rdfgraph

def queryRdfData(progname, options, prefixes, query, bindings):
    """
    Submit query against RDF data.
    Result is tuple of status and dictionary/list structure suitable for JSON encoding.
    """
    rdfgraph = getRdfData(options)
    if not rdfgraph:
        print "%s: Could not read RDF data (use -r <file> or supply RDF on stdin)"%progname
        return (2, None)
    query = prefixes + query
    resps = [rdfgraph.query(query, initBindings=b) for b in bindings['results']['bindings']]
    #for b in bindings['results']['bindings']:
    #    resp = rdfgraph.query(query, initBindings=b)
    #    resps.append(resp)
    res = { "head": {} }
    if resps[0].type == 'ASK':
        res["boolean"] = any([ r.askAnswer for r in resps ])
        return (0 if res["boolean"] else 1, res)
    elif resps[0].type == 'SELECT':
        res["head"]["vars"] = resps[0].vars
        res["results"] = {}
        res["results"]["bindings"] = [ bindingToJSON(b) for r in resps for b in r.bindings ]
        return (0 if len(res["results"]["bindings"]) > 0 else 1, res)
    elif resps[0].type == 'CONSTRUCT':
        res = rdflib.graph.ReadOnlyGraphAggregate( [r.graph for r in resps] )
        return (0 if len(res) > 0 else 1, res)
    else:
        assert False, "Unexpected query response type %s"%resp.type
    return (2, None)

def querySparqlEndpoint(progname, options, prefixes, query, bindings):
    """
    Issue SPARQL query to SPARQL HTTP endpoint.
    Requests either JSON or RDF/XML depending on query type.
    Returns JSON-like dictionary/list structure or RDF graph, depending on query type.
    These are used as basis for result formatting by outputResult function
    """
    query = prefixes + query
    resulttype = "application/RDF+XML"
    resultjson = False
    querytype  = queryType(query) 
    if querytype in ["ASK", "SELECT"]:
        # NOTE application/json doesn't work with Fuseki
        # See: http://gearon.blogspot.co.uk/2011/09/sparql-json-after-commenting-other-day.html
        resulttype = "application/sparql-results+json"
        resultjson = True
    sc = SparqlHttpClient(endpointuri=options.endpoint)
    ((status, reason), result) = sc.doQueryPOST(query, accept=resulttype, JSON=resultjson)
    if status != 200:
        assert False, "Error from SPARQL query request: %i %s"%(status, reason)
    status = 1
    if querytype == "SELECT":
        result['results']['bindings'] = parseJsonBindings(result['results']['bindings'])
        result['results']['bindings'] = joinBindingsToJSON(
                                            result['results']['bindings'], 
                                            bindings['results']['bindings'])
        if result['results']['bindings']: status = 0
    elif bindings:
        assert False, "Can't use supplied bindings with endpoint query other than SELECT"
    elif querytype == "ASK":
        # Just return JSON from Sparql query
        if result['boolean']: status = 0
    else:
        # return RDF
        rdfgraph = rdflib.Graph()
        try:
            # Note: declaring xml prefix in SPAQL query can result in invalid XML from Fuseki (v2.1)
            # See: https://issues.apache.org/jira/browse/JENA-24
            rdfgraph.parse(data=result)
            result = rdfgraph   # Return parsed RDF graph
            if len(result) > 0: status = 0
        except Exception, e:
            assert False, "Error parsing RDF from SPARQL endpoint query: "+str(e)
    return (status, result)

def outputResult(progname, options, result):
    outstr = sys.stdout
    if options.output and options.output != "-":
        print "Output to other than stdout not implemented"
    if isinstance(result, rdflib.Graph):
        result.serialize(destination=outstr, format="pretty-xml", base=None)
    elif isinstance(result, str):
        outstr.write(result)
    else:
        outstr.write(json.dumps(result))
        outstr.write("\n")
    return

def run(configbase, options, args):
    status   = 0
    progname = os.path.basename(args[0])
    query    = getQuery(options, args)
    if not query:
        print "%s: Could not determine query string (need query argument or -q option)"%progname
        print "Run '%s --help' for more information"%progname
        return 2
    prefixes = getPrefixes(options)
    if not prefixes:
        print "%s: Could not determine query prefixes"%progname
        print "Run '%s --help' for more information"%progname
        return 2
    bindings = getBindings(options)
    if not bindings:
        print "%s: Could not determine incoming variable bindings"%progname
        print "Run '%s --help' for more information"%progname
        return 2
    if options.endpoint:
        (status,result) = querySparqlEndpoint(progname, options, prefixes, query, bindings)
    else:
        (status,result) = queryRdfData(progname, options, prefixes, query, bindings)
    if result:
        outputResult(progname, options, result)
    return status

def parseCommandArgs(argv):
    """
    Parse command line arguments
    
    argv -- argument list from command line
    
    Returns a pair consisting of options specified as returned by
    OptionParser, and any remaining unparsed arguments.
    """
    # create a parser for the command line options
    parser = optparse.OptionParser(
                usage="%prog [options] [query]\n%prog --help  for option summary",
                description="A sparql query client, designed to be used as a filter in a command pieline. "+
                            "Pipelined data can be RDF or query variable binding sets, depending on the options used.",
                version="%prog "+asqc_settings.VERSION)
    parser.add_option("-q", "--query",
                      dest="query", 
                      help="URI or filename of resource containing query to execute. "+
                           "If not present, query must be supplied as command line argument.")
    parser.add_option("-p", "--prefix",
                      dest="prefix",
                      default="~/.asqc-prefixes",
                      help="URI or filename of resource containing query prefixes "+
                           "(default %default)")
    parser.add_option("-b", "--bindings",
                      dest="bindings",
                      default=None,
                      help="URI or filename of resource containing incoming query variable bindings "+
                           "(default none). "+
                           "Specify '-' to use stdin. "+
                           "This option works for SELECT queries only when accessing a SPARQL endpoint.")
    parser.add_option("-r", "--rdf-input",
                      action="append",
                      dest="rdf_data",
                      default=None,
                      help="URI or filename of RDF resource to query "+
                           "(default stdin or none). "+
                           "May be repeated to merge multiple input resources. "+
                           "Specify '-' to use stdin.")
    parser.add_option("-e", "--endpoint",
                      dest="endpoint",
                      default=None,
                      help="URI of SPARQL endpoint to query ")
    parser.add_option("-o", "--output",
                      dest="output",
                      default='-',
                      help="URI or filename of RDF resource for output "+
                           "(default stdout)."+
                           "Specify '-'to use stdout.")
    parser.add_option("-t", "--type",
                      dest="query_type",
                      default=None,
                      help="Type of query output: SELECT (variable bindings, CONSTRUCT (RDF) or ASK (status)")
    parser.add_option("-v", "--verbose",
                      action="store_true", 
                      dest="verbose", 
                      default=False,
                      help="display verbose output")
    # parse command line now
    (options, args) = parser.parse_args(argv)
    if len(args) < 1: parser.error("No command present")
    if len(args) > 2: parser.error("Too many arguments present: "+repr(args))
    return (options, args)

def runCommand(configbase, argv):
    """
    Run program with supplied configuration base directory, Base directory 
    from which to start looking for research objects, and arguments.
    
    This is called by main function (below), and also by test suite routines.
    
    Returns exit status.
    """
    log.debug("runCommand: configbase %s, argv %s"%(configbase, repr(argv)))
    (options, args) = parseCommandArgs(argv)
    status = 2
    if options:
        status  = run(configbase, options, args)
    return status

def runMain():
    """
    Main program transfer function for setup.py console script
    """
    configbase = os.path.expanduser("~")
    return runCommand(configbase, sys.argv)

if __name__ == "__main__":
    """
    Program invoked from the command line.
    """
    # main program
    status = runMain()
    sys.exit(status)

#--------+---------+---------+---------+---------+---------+---------+---------+



# $Id: TestNetUtils.py 1047 2009-01-15 14:48:58Z graham $
#
# Unit testing for WebBrick library functions (Functions.py)
# See http://pyunit.sourceforge.net/pyunit.html
#

import sys
import os
import unittest

# Adjust this to match test environment:
#testAllNetMasks = ["10.0.0.0/8", "193.123.195.0/24", "193.123.216.64/26" ]
testAllNetMasks = ["193.123.216.64/26" ]

sys.path.append("../..")
from MiscLib.NetUtils import *

class TestNetUtils(unittest.TestCase):
    def setUp(self):
        return

    def tearDown(self):
        return

    # Actual tests follow

    def testIpAdrStrToInt(self):
        self.assertEqual( ipAdrStrToInt( "1.2.3.4" ), 16909060 )

    def testAddBroadcastBits(self):
        self.assertEqual( addBroadcastBits( 0x01020304, 24 ), 0x010203FF )

    def testGetBroadcastAddressI(self):
        self.assertEqual( getBroadcastAddressI( "1.2.3.4","16"), 0x0102FFFF )

    def testGetBroadcastAddress1(self):
        self.assertEqual( getBroadcastAddress( "1.2.3.4/16"), "1.2.255.255" )

    def testGetBroadcastAddress2(self):
        self.assertEqual( getBroadcastAddress( "1.2.3.4/20"), "1.2.15.255" )

    # Tests for methods representing addresses as integer lists

    def testParseIpAdrs(self):
        self.assertEqual(parseIpAdrs("193.123.216.121"), [193,123,216,121])

    def testParseNetAdrs(self):
        self.assertEqual(parseNetAdrs("193.123.216.64/26"), ([193,123,216,64],26))

    def testMkNetMask1(self):
        n = parseNetAdrs("193.123.216.64/26")
        self.assertEqual(mkNetMask(*n), [255,255,255,192])

    def testMkNetMask2(self):
        n = parseNetAdrs("193.123.216.64/26")
        self.assertEqual(mkNetMask(n), [255,255,255,192])

    def testMkBroadcastAddress1(self):
        n = parseNetAdrs("193.123.216.64/26")
        self.assertEqual(mkBroadcastAddress(*n), [193,123,216,127])

    def testMkBroadcastAddress2(self):
        n = parseNetAdrs("193.123.216.64/26")
        self.assertEqual(mkBroadcastAddress(n), [193,123,216,127])
        
    def testFormatIpAdrs(self):
        self.assertEqual(formatIpAdrs([193,123,216,64]), "193.123.216.64")
        
    def testIpInNetwork1(self):
        n = parseNetAdrs("193.123.216.64/26")
        i = parseIpAdrs("193.123.216.121")
        b = mkBroadcastAddress(*n)
        x = parseIpAdrs("193.123.216.200")
        assert ipInNetwork(i,*n)
        assert ipInNetwork(b,*n)
        assert not ipInNetwork(x,*n)
        
    def testIpInNetwork2(self):
        n = parseNetAdrs("193.123.216.64/26")
        i = parseIpAdrs("193.123.216.121")
        b = mkBroadcastAddress(n)
        x = parseIpAdrs("193.123.216.200")
        assert ipInNetwork(i,n)
        assert ipInNetwork(b,n)
        assert not ipInNetwork(x,n)

    def testParseMacAdrs(self):
        self.assertEqual(parseMacAdrs("01:34:67:9a:BC:eF"), [1,52,103,154,188,239])
        
    def testFormatMacAdrs(self):
        self.assertEqual(formatMacAdrs([1,52,103,154,188,239],sep='-'), "01-34-67-9A-BC-EF")
        
    def testGetHostIpsAndMask(self):
        # getHostIpsAndMask doesn't work for networks witjh non-standard netmasks
        results = getHostIpsAndMask()
        for v in testAllNetMasks:
            self.assert_( v in results, "Not found: %s in %s"%(v,results) )

# Code to run unit tests directly from command line.
# Constructing the suite manually allows control over the order of tests.
def getTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(TestNetUtils("testIpAdrStrToInt"))
    suite.addTest(TestNetUtils("testAddBroadcastBits"))
    suite.addTest(TestNetUtils("testGetBroadcastAddressI"))
    suite.addTest(TestNetUtils("testGetBroadcastAddress1"))
    suite.addTest(TestNetUtils("testGetBroadcastAddress2"))
    suite.addTest(TestNetUtils("testParseIpAdrs"))
    suite.addTest(TestNetUtils("testParseNetAdrs"))
    suite.addTest(TestNetUtils("testMkNetMask1"))
    suite.addTest(TestNetUtils("testMkNetMask2"))
    suite.addTest(TestNetUtils("testMkBroadcastAddress1"))
    suite.addTest(TestNetUtils("testMkBroadcastAddress2"))
    suite.addTest(TestNetUtils("testFormatIpAdrs"))
    suite.addTest(TestNetUtils("testIpInNetwork1"))
    suite.addTest(TestNetUtils("testIpInNetwork2"))
    suite.addTest(TestNetUtils("testParseMacAdrs"))
    suite.addTest(TestNetUtils("testFormatMacAdrs"))
    # suite.addTest(TestNetUtils("testGetHostIpsAndMask"))
    return suite

if __name__ == "__main__":

    logging.basicConfig(level=logging.DEBUG)

    # unittest.main()
    runner = unittest.TextTestRunner()
    runner.run(getTestSuite())
    


# $Id: TestAll.py 1047 2009-01-15 14:48:58Z graham $
#
# Unit testing for WebBrick library functions (Functions.py)
# See http://pyunit.sourceforge.net/pyunit.html
#

import sys, unittest, logging

# Add main library directory to python path
sys.path.append("../..")

import TestTestUtils
import TestFunctions
import TestCombinators
import TestDomHelpers
import TestScanFiles
import TestScanDirectories
import TestNetUtils
import TestSuperGlobal

# Code to run unit tests from all library test modules
def getTestSuite(select="unit"):
    suite = unittest.TestSuite()
    suite.addTest(TestTestUtils.getTestSuite(select=select))
    suite.addTest(TestFunctions.getTestSuite())
    suite.addTest(TestCombinators.getTestSuite())
    suite.addTest(TestDomHelpers.getTestSuite())
    suite.addTest(TestScanFiles.getTestSuite())
    suite.addTest(TestScanDirectories.getTestSuite())
    suite.addTest(TestNetUtils.getTestSuite())
    suite.addTest(TestSuperGlobal.getTestSuite())
    return suite

from MiscLib import TestUtils

if __name__ == "__main__":
    TestUtils.runTests("TestAll", getTestSuite, sys.argv)

# End.
